#!/bin/sh
# Test "rm -i".

# Copyright (C) 1997-2022 Free Software Foundation, Inc.

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.

. "${srcdir=.}/tests/init.sh"; path_prepend_ ./src
print_ver_ rm

t=t
mkdir -p $t || framework_failure_
echo > $t/a || framework_failure_
test -f $t/a || framework_failure_

echo y > $t/in-y
echo n > $t/in-n

rm -i $t/a < $t/in-n > /dev/null 2>&1 || fail=1
# The file should not have been removed.
test -f $t/a || fail=1

rm -i $t/a < $t/in-y > /dev/null 2>&1 || fail=1
# The file should have been removed this time.
test -f $t/a && fail=1

rm -rf $t

Exit $fail
